// YCAM AMP SHIELD PIN LAYOUT
// A0 - リファレンス用
// A1 - 測定値用
// A3 - 温度用
// A4,A5 - RTC通信用(I2C)

// 温度 = ((temp - 100) / 10.0) - 40.0;
#include <stdlib.h>
#include <Time.h>
#include <Wire.h>
#include <DS1307RTC.h>
#include <SD.h>

#define AVG_COUNT  (32)
#define SLEEP_MS   (1000)
#define VCC        (5.0)
#define ADCUNIT    ((VCC / 1024.0) * 1000.0) // mV
#define TIME_HEADER  "T"

const int chipSelect = 8;

void setup() {
  Serial.begin(9600);
  SD.begin(chipSelect);
  Wire.begin();
  while (!Serial) ; // Needed for Leonardo only
  setSyncProvider(RTC.get);   // the function to get the time from the RTC
  if (timeStatus() != timeSet) {
     Serial.println("Unable to sync with the RTC");
  } else {
     Serial.println("RTC has set the system time");      
  }
}


void loop() {
  if (Serial.available()) {
    time_t t = processSyncMessage();
    if (t != 0) {
      RTC.set(t);
      setTime(t);          
    }
  }

  float a0, a1, a2, volt;
  char buffer[10];
  String dataString;

  a0 = 0;
  a1 = 0;
  a2 = 0;

  // throw away 1st ADC value.
  analogRead(0);
  analogRead(1);

  // read analog input.
  for(int i = 0; i < AVG_COUNT; i++) {
    a0 += analogRead(0);
    a1 += analogRead(1);
  }

  volt = (float)((((a1 - a0) / AVG_COUNT) * (VCC / 1024.0)) / 5.0);
  
  String voltage = dtostrf(volt, 8, 5, buffer);
  voltage.replace(" ", "");

  dataString = "\"";
  dataString += (String)year();
  dataString += "-";
  dataString += (String)month();
  dataString += "-";
  dataString += (String)day();
  dataString += " ";
  dataString += (String)hour();
  dataString += ":";
  dataString += (String)minute();
  dataString += ":";
  dataString += (String)second();
  dataString += "\",\"";
  dataString += voltage;
  dataString += "\"";
  Serial.print("DATASTRING: ");
  Serial.println(dataString);
  
  //File dataFile = SD.open("datalog.txt", FILE_WRITE);
  File dataFile = SD.open("datalog.csv", FILE_WRITE);
  if (dataFile) {
    dataFile.println(dataString);
    dataFile.close();
  } else {
    Serial.println("error opening datalog.csv");
  }   

  delay(1000);
}

unsigned long processSyncMessage() {
  unsigned long pctime = 0L;
  const unsigned long DEFAULT_TIME = 1357041600; // Jan 1 2013 

  if(Serial.find(TIME_HEADER)) {
     pctime = Serial.parseInt();
     return pctime;
     if( pctime < DEFAULT_TIME) { // check the value is a valid time (greater than Jan 1 2013)
       pctime = 0L; // return 0 to indicate that the time is not valid
     }
  }
  return pctime;
}

